/*
 * misc. BIOS related defines
 *
 * Copyright (c) 1999-2000 VM Labs, Inc. All rights reserved.
 * Confidential and Proprietary Information of VM Labs, Inc.
 * These materials may be used or reproduced solely under
 * an express written license from VM Labs, Inc.
 *
 */

/* $Id: bios.h,v 1.15 2000/10/27 21:41:59 ersmith Exp $ */

#ifndef _BIOS_H
#define _BIOS_H

#ifdef __cplusplus
extern "C" {
#endif

/************************************************
 Bios information structure and function
 ************************************************/

struct BiosInfo {
    unsigned short major_version;
    unsigned short minor_version;
    unsigned short vm_revision;
    unsigned short oem_revision;
    char *info_string;
    char *date_string;
    unsigned int HAL_version;
};

struct BiosInfo *_BiosGetInfo(void);

/************************************************
 Interrupt setting functions
 ************************************************/

void *_IntSetVector(int which, void *newvector);
void *_IntGetVector(int which);

/* defines for interrupt vector numbers */
#define kIntrVideo      31
#define kIntrSystimer1  30
#define kIntrSystimer0  29
#define kIntrGPIO       28
#define kIntrAudio      27
#define kIntrHost       26
#define kIntrDebug      25
#define kIntrMBDone     24
#define kIntrDCTDone    23
#define kIntrIIC        20
#define kIntrSystimer2  16
#define kIntrCommXmit    5
#define kIntrCommRecv    4
#define kIntrSoftware    1
#define kIntrException   0


/************************************************
 Parental controls
 ************************************************/
void _SetParentalControl(int level);
int _GetParentalControl(void);

/************************************************
 Game loading
 ************************************************/
void _LoadGame(const char *filename);

/************************************************
 Patching jump table
 ************************************************/
void *_PatchJumptable(void *entry, void *function);

/************************************************
 Memory allocation
 ************************************************/
void _MemInit(int dokernel);
void _MemAdd(unsigned long base, unsigned long size, unsigned flags);
void *_MemAlloc(unsigned long size, unsigned align, unsigned flags);
void _MemFree(void *);
void *_MemLocalScratch(int *);

/* flags for _MemAlloc and _MemAdd */
#define kMemSDRAM  1
#define kMemSysRam 2
#define kMemKernel 4

/************************************************
 Emergency broadcast messages
 ************************************************/

int _BiosPoll(char *msg);
#define kPollContinue    0
#define kPollSaveExit    1
#define kPollDisplayMsg  2
#define kPollPauseMsg    3

int _BiosPauseMsg(int rval, char *msg, void *framebuf);


/************************************************
 Other BIOS functions
 ************************************************/

void _BiosExit(int exitcode);
void _BiosReboot(void);

#ifdef __cplusplus
}
#endif


#include <nuon/mpe.h>
#include <nuon/comm.h>
#include <nuon/cache.h>
#include <nuon/audio.h>
#include <nuon/video.h>
#include <nuon/joystick.h>
#include <nuon/time.h>
#include <nuon/mediaio.h>


#endif


