/*
 * joytest -- test of joystick functions
 *
 * Copyright (c) 2000 VM Labs, Inc.
 * All rights reserved.
 * Confidential and Proprietary Information
 * of VM Labs, Inc.
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdarg.h>
#include <nuon/dma.h>
#include <nuon/mutil.h>
#include <nuon/msprintf.h>
#include <nuon/bios.h>

/*
 * defaults for things
 */
#define SCRNWIDTH 392
#define SCRNHEIGHT 240
#define DMA_XFER_TYPE 4

#define LEFT_MARGIN	(40)

#define JOY_DEAD_X	(16)
#define JOY_DEAD_Y	(16)

int dma_width = SCRNWIDTH;
int dma_height = SCRNHEIGHT;
int dma_xsize = (SCRNWIDTH/8)<<16;

#define FGCOLOR 0xc0808000  /* white */
#define BGCOLOR 0x60808000  /* grey */
#define BALLCOLOR 0xc0ff0000

#define XSIZE ((scrnwidth/8)<<16)
#define DMAFLAGS (DMA_PIXEL_WRITE | dma_xsize | (DMA_XFER_TYPE<<4) | (1<<11))

#define SCREEN ((void *)0x40000000)


/*
 * erase the screen by drawing 8x8 rectangle blocks; not perfectly efficient,
 * but pretty good
 * NOTE: assumes that the screen height and width are multiples of 8!
 */

void ClearScreen()
{
long x, y;

    for (x = 0; x < dma_width; x += 8)
	{
		for (y = 0; y < dma_height; y += 8)
		{
			_raw_plotpixel(DMAFLAGS, SCREEN, (8<<16)|x, (8<<16)|y, BGCOLOR);
		}
    }
}

#define SPRITE_WIDTH 4
#define SPRITE_HEIGHT 4

/*
 * draws a sprite centered at (xpos, ypos)
 * assumes that higher level code has already checked for
 * bounding conditions
 */
void DrawSprite(int xpos, int ypos)
{
    xpos -= SPRITE_WIDTH/2;
    ypos -= SPRITE_WIDTH/2;

    _raw_plotpixel(DMAFLAGS, SCREEN, (SPRITE_WIDTH<<16)|xpos, (SPRITE_HEIGHT<<16)|ypos, BALLCOLOR);
}


void adjust_for_joystick_deadzone( int x_deadzone, int y_deadzone, int *joy_x, int *joy_y )
{
register int x, y;

	x = *joy_x;
	y = *joy_y;

	if( x > 0  && x < x_deadzone )
		x = 0;
	else if( x > 0 )
		x -= x_deadzone;

	if( x < 0 && x > (-x_deadzone) )
		x = 0;
	else if( x < 0 )
		x += x_deadzone;

	if( y > 0 && y < y_deadzone )
		y = 0;
	else if( y > 0 )
		y -= y_deadzone;

	if( y < 0 && y > (-y_deadzone) )
		y = 0;
	else if( y < 0 )
		y += y_deadzone;

	*joy_x = x;
	*joy_y = y;
}

int main(void)
{
int deltax, deltay, xpos, ypos;
int minx, miny, maxx, maxy;
int has_analogstick;
char buf[SPRINTF_MAX];

	VidSetup(SCREEN, DMAFLAGS, dma_width, dma_height, 0);

    xpos = dma_width/2;
    ypos = dma_height/2;
    minx = 16;
    miny = 16;
    maxx = dma_width - (minx+1);
    maxy = dma_height - (miny+1);

	for(;;)
	{
		/* erase old sprite and text */
		ClearScreen();

		msprintf(buf, "Buttons: %08x", _Controller[1].buttons );
		DebugWS(DMAFLAGS, SCREEN, LEFT_MARGIN, 30, FGCOLOR, buf);
		
		/* update position based on analog joystick values */
		/* these are signed 8 bit values */
		has_analogstick = _Controller[1].properties & CTRLR_ANALOG1;

		if( has_analogstick )
		{
			deltax = JoyXAxis(_Controller[1]);
			deltay = JoyYAxis(_Controller[1]);
	
			/* we want "y" to increase to the bottom of the screen */
			deltay = -deltay;

			adjust_for_joystick_deadzone( JOY_DEAD_X, JOY_DEAD_Y, &deltax, &deltay );
			
			/* update positions */
			xpos += deltax / 4;
			ypos += deltay / 4;
		
			// Make sure sprite is still on screen
			// Wrap around to other side when hit boundary
			if (xpos < minx)
				xpos = maxx;
			else if (xpos > maxx)
				xpos = minx;
		
			if (ypos < miny)
				ypos = maxy;
			else if (ypos > maxy)
				ypos = miny;
		
			/* update sprite */
			DrawSprite(xpos, ypos);

			msprintf( buf, "X: %d", deltax );
			DebugWS(DMAFLAGS, SCREEN, LEFT_MARGIN, 45, FGCOLOR, buf);
			msprintf( buf, "Y: %d", deltay );
			DebugWS(DMAFLAGS, SCREEN, LEFT_MARGIN, 60, FGCOLOR, buf);
	
			DebugWS(DMAFLAGS, SCREEN, LEFT_MARGIN, 160, FGCOLOR, "This program implements a deadzone" );
			DebugWS(DMAFLAGS, SCREEN, LEFT_MARGIN, 175, FGCOLOR, "for the analog joystick in order to" );
			DebugWS(DMAFLAGS, SCREEN, LEFT_MARGIN, 190, FGCOLOR, "ignore tiny movements or slightly" );
			DebugWS(DMAFLAGS, SCREEN, LEFT_MARGIN, 205, FGCOLOR, "uncentered positions." );
		}
		else
		{
			DebugWS(DMAFLAGS, SCREEN, LEFT_MARGIN, 100, FGCOLOR, "No analog joystick detected" );
		}
	
		/* wait for vblank */
		_VidSync(1);
	}

    return 0;
}

