/* Copyright (c) 1995-2001, VM Labs, Inc., All rights reserved.
 * Confidential and Proprietary Information of VM Labs, Inc
 * 1/25/01 kml
 *
 * NuonGamesChannelManager is a derived class of NuonChannelManager
 * which uses BIOS calls for hardware-dependent operations
 */

#ifndef NuonGamesChannelManager_DEFINED
#define NuonGamesChannelManager_DEFINED

#include "NuonChannelManager.h"



// channel manager for games (bios-controlled)
class NuonGamesChannelManager : public NuonChannelManager {

	// system resources
	static mmlSysResources mSysRes;

	// device contexts
	mmlGC			*mpGcMain;
	mmlGC			*mpGcOsd;

	// BIOS channel and display constructs
	VidDisplay		mDisplay;
	VidChannel		*mpChannelMain;
	VidChannel		*mpChannelOsd;

	
// returns nonzero on failure, 0 on success
// receives both depth and pixel format
// private method
virtual int InitPixmaps (mmlAppPixmap *sP, void *appMem, 
		 int wide, int high, int depth, mmlPixFormat pix, int numBuffers);

public:
	NuonGamesChannelManager();
	virtual ~NuonGamesChannelManager();

	// accessor for system resources: actual data type returned is a function of the
	// platform
	virtual const void *GetSystemResources (void) { return (void*) &mSysRes;  }

	// mmlInitGC()
	virtual int InitChannel (const unsigned channel = kChOsd);

	virtual void* GetLocalRamAdr (void) { return mSysRes.intDataAdr; }

	// Show and Hide take an (optional) argument of channels
	// For Show and Hide, setting a channels flag for an inactive (uninitialized) channel 
	// is acknowledged in the return status but otherwise ignored as follows:
	// returns 0 on success, <0 on failure, >0 on one or both requested channels ignored
	// Config checks mmlDisplayPixmap::dmaFlags to verify pixmap can be displayed on
	// the selected channel
	
	// configure a channel before a call to Show()
	// this is where the check for allowed pixel format(s) is done
	virtual int ConfigChannel (mmlDisplayPixmap *sP, const unsigned channel = kChOsd);

	// show channels that have been configured with a call to ConfigChannel
	virtual int Show (const unsigned channels = (kChOsd | kChMain), 
						 const int alpha = 0);
	// hide channels
	virtual int Hide (const unsigned channels = (kChOsd | kChMain));

	// surrender channels
	virtual int ReleaseChannel (const unsigned channels = (kChOsd | kChMain));

	// Loads the OSD hardware Clut
	virtual void SetClut (const NuonYccColorTable *clut);
	
	// Call with a new whichBuf to do a buffer flip (buffers all attached to the mmlDisplayPixmap
	// whichBuf must be less than the number of buffers allocated in the call to InitPixmaps
	virtual void SetDisplayFrame (mmlDisplayPixmap *sP, 
						 const unsigned channel = kChOsd, const unsigned whichBuf = 0);
	// Allocate pixmap(s)
	// returns mml status or Windows equiv (check return type)
	// uses the address specified in appMem or dispMem if not null
	
	// returns nonzero on failure, 0 on success
	// specifying depth defaults pixel format to RGBA
	virtual int InitPixmaps (mmlAppPixmap *sP, void *appMem, int wide, int high, 
						 int depth, int numBuffers = 1);

	virtual int InitPixmaps (mmlAppPixmap *sP, void *appMem, int wide, int high,
						 mmlPixFormat pix, int numBuffers = 1);

	// returns nonzero on failure, 0 on success
	// specifying depth defaults pixel format to YCCA
	virtual int InitPixmaps( mmlDisplayPixmap* sP, void *dispMem, int wide, int high, 
						 int depth, int numBuffers = 1);

	virtual int InitPixmaps( mmlDisplayPixmap* sP, void *dispMem, int wide, int high,
						 mmlPixFormat pix, int numBuffers = 1);

	// these do the same thing, but free the caller from needing to cast the Pixmap type
	virtual void ReleasePixmaps (mmlAppPixmap *sp, int numBuffers = 1);
	virtual void ReleasePixmaps (mmlDisplayPixmap *sp, int numBuffers = 1);

	// raster buffer (copy) operations
	virtual void CopyDisplay(mmlDisplayPixmap *sp, hsIntRect *rect, int x, int y, 
							const unsigned channel = kChOsd);
	virtual void CopyRect8 (mmlAppPixmap *appP, mmlDisplayPixmap *sP,
							const hsIntRect *rect, const unsigned channel = kChOsd);
	virtual void CopyRect16 (mmlAppPixmap *appP, mmlDisplayPixmap *sP,
							const hsIntRect *rect, const unsigned channel = kChOsd);
	virtual void CopyRect32 (mmlAppPixmap *appP, mmlDisplayPixmap *sP,
							const hsIntRect *rect, const unsigned channel = kChOsd);


#if defined (PRINT_DEBUG)
	virtual void PrintState (char *objectName);
#endif
};

#endif  //NuonGamesChannelManager_DEFINED



