/*
   Copyright (c) 1995-1999, VM Labs, Inc., All rights reserved.
   Confidential and Proprietary Information of VM Labs, Inc.
   These materials may be used or reproduced solely under an express
   written license from VM Labs, Inc.
 */
/* rwb 6/24/99
 * MRP function for copy from e888alpha AppPixmap to 655 Display Pixmap
 * bot leftCol's must be even, and rowLength must be even.
 */

#include <nuon/mml2d.h>
#include "mrpproto.h"
#include "pixmacro.h"
#include "parblock.h"
#include <stddef.h>

typedef struct Copy32to16ParamBlock Copy32to16ParamBlock;
struct Copy32to16ParamBlock
{
	void*	srcBufferAdr;
	int		srcByteWidth;	// width of Pixmap (in bytes)				
	int		srcLeftCol;			
	int		srcTopRow;
	void*	destBufferAdr;
	int		destFlags;		// must be pixtype 8 and Write	
	int		destLeftCol;			
	int		destTopRow;
	int		rowLength;
	int		numRows;			
};


/*-------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------*/
mrpStatus Copy32to16(int environs, Copy32to16ParamBlock* parBlockP, int arg2, int arg3 ) 
{
 	odmaCmdBlock*			odmaP;
 	mdmaCmdBlock*			mdmaP;
 	Copy32to16ParamBlock*	parP;
 	uint8*					tileBase;
 	int						srcAdr;
 	int						writeRow;
 	int						endRow;
 	int						parSizeLongs;
	int						mrpsetupstatus;

 	/* Set up local dtram & read in parameter block or  */
 	parSizeLongs = (sizeof(Copy32to16ParamBlock)+3)>>2;
 	mrpsetupstatus = 	mrpSetup
						(
							environs,
							parSizeLongs,
							&odmaP,
							&mdmaP,
							(int**)&parP,
							&tileBase,
							NULL 
						);
 	if (mrpsetupstatus)
	{
	  	mrpSysRamMove
		(
			parSizeLongs,
			(char*)parP,
			(char*)parBlockP,
			odmaP,
			kSysReadFlag,
			kWaitFlag
		);
	}
	else 
	{
		parP = parBlockP;
	}

 #ifdef BB /* another hack for maui */
 	_SetLocalVar( parP->destBufferAdr , kGraphicsBase);
 #endif

	srcAdr =	  _GetLocal( parP->srcBufferAdr )
				+ _GetLocal( parP->srcByteWidth ) * _GetLocal( parP->srcTopRow )
				+ (_GetLocal(parP->srcLeftCol)<<2);
	writeRow = _GetLocal( parP->destTopRow );
	endRow = writeRow + _GetLocal( parP->numRows );
 	MRP_DmaWait( kodmactl );

	_SetLocalVar( odmaP->dramAdr, tileBase );
  	MRP_DmaWait( kmdmactl );
	_SetLocalVar( mdmaP->dramAdr, tileBase );
	_SetLocalVar( mdmaP->sdramAdr, _GetLocal( parP->destBufferAdr ));
	_SetLocalVar( mdmaP->flags, _GetLocal( parP->destFlags));
	while( writeRow < endRow )
	{
		int pixToDo = _GetLocal(parP->rowLength);
		int dstStart = _GetLocal(parP->destLeftCol);
		int srcStart = srcAdr;
		while (pixToDo > 0)
		{
			int segSize = pixToDo > kMaxLongs ? kMaxLongs : pixToDo;
			_SetLocalVar( odmaP->sysAdr, srcStart );
			_SetLocalVar( odmaP->flags, 0x2000 | (segSize<<16));
			MRP_DmaDo( kodmactl, odmaP, 1 );
			_SetLocalVar( mdmaP->xDesc, segSize<<16 | dstStart );
			_SetLocalVar( mdmaP->yDesc, 1<<16 | writeRow );
			MRP_DmaDo( kmdmactl, mdmaP, 1 );
			dstStart += segSize;
			srcStart += (segSize<<2);
			pixToDo -= segSize;
		}
		++writeRow;
		srcAdr += _GetLocal( parP->srcByteWidth );
	}
	return eFinished;
}

/*-------------------------------------------------------------------------------------------------
	CopyRectFast
	- Copy a block of pixels from sysRam to SDRAM.
	- No color conversion or expansion is done.
	- No scaling is done.
	- Format Translations are:
		e888Alpha	to	e888Alpha
		e888Alpha	to	e655
		e655		to	e655
		e8Clut		to	e8Clut 
	- Source left column must start on long word boundary
	- Destination left column must start on long word boundary
	- The width of the source block in bytes must be a multiple of 4.
-------------------------------------------------------------------------------------------------*/
mrpStatus CopyRectFast( int environs, CopyRectFastParamBlock* parBlockP, int arg2, int arg3 ) 
{
 	odmaCmdBlock*			odmaP;
 	mdmaCmdBlock*			mdmaP;
 	CopyRectFastParamBlock*	parP;
 	uint8*					tileBase;
 	int						srcAdr;
 	int						writeRow;
 	int						endRow;
 	int						maxPix;
 	int						parSizeLongs = (sizeof(CopyRectFastParamBlock)+3) >> 2;
	int						mrpSetupStat;
	int						expand = 0;

 	/* Set up local dtram & read in parameter block or  */
	mrpSetupStat = mrpSetup( environs, parSizeLongs, &odmaP, &mdmaP, (int**)&parP, &tileBase, NULL );
	if (mrpSetupStat)
	{
		mrpSysRamMove( parSizeLongs, (char*)parP, (char*)parBlockP, odmaP, kSysReadFlag, kWaitFlag );
	}
	else 
	{
		parP = parBlockP;	
	}
 /* another hack for maui */
 #ifdef BB
 	_SetLocalVar( parP->destBufferAdr , kGraphicsBase);
 #endif
	srcAdr = _GetLocal( parP->srcBufferAdr )
	       + _GetLocal( parP->srcByteWidth ) * _GetLocal( parP->srcTopRow )
	       + (_GetLocal(parP->srcLeftCol) << (2 - _GetLocal( parP->srcPixShift ))  );
	writeRow = _GetLocal( parP->destTopRow );
	endRow = writeRow + _GetLocal( parP->numRows );
	if( _GetLocal( parP->srcPixShift ) == 1 && (_GetLocal( parP->destFlags ) & 0xF0) == 0x40 )
		expand = 1;
 	MRP_DmaWait( kodmactl );
	_SetLocalVar( odmaP->dramAdr, tileBase );

  	MRP_DmaWait( kmdmactl );
	_SetLocalVar( mdmaP->dramAdr , tileBase );
	_SetLocalVar( mdmaP->sdramAdr , _GetLocal( parP->destBufferAdr ));
	_SetLocalVar( mdmaP->flags, _GetLocal( parP->destFlags));
	maxPix = kMaxLongs << _GetLocal( parP->srcPixShift );
	while( writeRow < endRow )
	{
		int pixToDo = _GetLocal(parP->rowLength);
		int dstStart = _GetLocal(parP->destLeftCol);
		int srcStart = srcAdr;
		int segSize = maxPix;
		_SetLocalVar( mdmaP->yDesc, 1<<16 | writeRow );
		_SetLocalVar( odmaP->flags, 0x2000 | ((segSize>>_GetLocal(parP->srcPixShift))<<16));
		while( pixToDo > 0 )
		{
			if( pixToDo < maxPix )
			{
				segSize = pixToDo;
				_SetLocalVar( odmaP->flags, 0x2000 | ((segSize>>_GetLocal(parP->srcPixShift))<<16));
			}	
			_SetLocalVar( odmaP->sysAdr, srcStart );
			MRP_DmaDo( kodmactl, odmaP, 1 );
			if( expand )
			{
				uint16* p = (uint16*)tileBase + segSize;
				uint32* q = (uint32*)tileBase + segSize;
				int j;
				for(j=0; j<segSize; ++j )
				{
					int val = *--p;
					int y = val & 0xFC00;
					int cr = val & 0x3E0;
					int cb = val & 0x1F;
					*--q = (y<<16) | (cr<<14) | (cb<<11);
				}
			}
			_SetLocalVar( mdmaP->xDesc, segSize<<16 | dstStart );
			MRP_DmaDo( kmdmactl, mdmaP, 1 );
			dstStart += segSize;
			srcStart += ((segSize>>_GetLocal(parP->srcPixShift))<<2);
			pixToDo -= segSize;
		}
		++writeRow;
		srcAdr += _GetLocal( parP->srcByteWidth );
	}
	return eFinished;
}

/*-------------------------------------------------------------------------------------------------
	CopyRGBFast
	- Copy a block of eRGB0555 pixels from sysRam and convert to YCC655 in SDRAM.
	- No scaling is done.
	- Transparent pixels are not supported
-------------------------------------------------------------------------------------------------*/
mrpStatus CopyRGBFast( int environs, CopyRectFastParamBlock* parBlockP, int arg2, int arg3 ) 
{
 	odmaCmdBlock*			odmaP;
 	mdmaCmdBlock*			mdmaP;
 	CopyRectFastParamBlock*	parP;
 	uint8*					tileBase;
 	int						srcAdr;
 	int						writeRow;
 	int						endRow;
 	int						parSizeLongs = (sizeof(CopyRectFastParamBlock)+3) >> 2;
	int						mrpSetupStat;

 	/* Set up local dtram & read in parameter block or  */
	mrpSetupStat = mrpSetup( environs, parSizeLongs, &odmaP, &mdmaP, (int**)&parP, &tileBase, NULL );
	if (mrpSetupStat)
	{
		mrpSysRamMove( parSizeLongs, (char*)parP, (char*)parBlockP, odmaP, kSysReadFlag, kWaitFlag );
	}
	else 
	{
		parP = parBlockP;	
	}
 /* another hack for maui */
 #ifdef BB
 	_SetLocalVar( parP->destBufferAdr , kGraphicsBase);
 #endif
	srcAdr = _GetLocal( parP->srcBufferAdr )
	       + _GetLocal( parP->srcByteWidth ) * _GetLocal( parP->srcTopRow )
	       + (_GetLocal(parP->srcLeftCol) << 1 );
	writeRow = _GetLocal( parP->destTopRow );
	endRow = writeRow + _GetLocal( parP->numRows );
 	MRP_DmaWait( kodmactl );
	_SetLocalVar( odmaP->dramAdr, tileBase );

  	MRP_DmaWait( kmdmactl );
	_SetLocalVar( mdmaP->dramAdr , tileBase );
	_SetLocalVar( mdmaP->sdramAdr , _GetLocal( parP->destBufferAdr ));
	_SetLocalVar( mdmaP->flags, _GetLocal( parP->destFlags));
	SetIndex( xybase, xyctl, rx ,ry, tileBase, 2<<20, 0, 1 )
	SetIndex( uvbase, uvctl, ru ,rv, tileBase, (4<<20) | (1<<28), 0, 1 )
	while( writeRow < endRow )
	{
		int pixToDo = _GetLocal(parP->rowLength);
		int dstStart = _GetLocal(parP->destLeftCol);
		int srcStart = srcAdr;
		_SetLocalVar( mdmaP->yDesc, 1<<16 | writeRow );
		_SetLocalVar( odmaP->flags, 0x2000 | (kMaxLongs<<16));
		copyfast16A( pixToDo, srcStart, dstStart, odmaP, mdmaP );
		++writeRow;
		srcAdr += _GetLocal( parP->srcByteWidth );
	}
	return eFinished;
}
