/*
   Copyright (c) 2000, VM Labs, Inc., All rights reserved.
   Confidential and Proprietary Information of VM Labs, Inc.
   These materials may be used or reproduced solely under an express
   written license from VM Labs, Inc.
 
   Written by Mike Fulton, VM Labs, Inc.
 
 */

#include "showjpeg.h"

////////////////////////////////////////////////////////////////////////////
// picture = pointer (long aligned) to source image data (32-bit)
// xoff & yoff = Destination screen target position
// imgwid & imght = Source image size
// wclip & hclip = maximum source coordinates for drawing image
////////////////////////////////////////////////////////////////////////////

#define DMA_SEGMENT_LENGTH	(64)	// Presumes that 1 pixel = 1 longword

void draw_picture(long *picture,int xoff, int yoff, int imgwid, int imght, int wclip, int hclip )
{
long *theScanline, scratchsize;
int xx, yy;
int segment_width;
int readflags;

	// Get address of internal memory scratch buffer to use for image buffer
	long *imgbuf = _MemLocalScratch((void *)&scratchsize);
	
	theScanline = picture;

	for( yy = 0; yy < hclip; yy++ )
	{
		for( xx = 0; xx < wclip; xx += DMA_SEGMENT_LENGTH )
		{
			segment_width = imgwid - xx;
			if( segment_width > DMA_SEGMENT_LENGTH )
				segment_width = DMA_SEGMENT_LENGTH;

			readflags = (1<<13) | (segment_width << 16);	// flags = READ, # longs

			// Read section
			_DMALinear( readflags, &theScanline[xx], (void *)imgbuf );
	
			// Write section
			_DMABiLinear(	gl_screenbuffers[gl_drawbuffer].dmaFlags,
							(void *)gl_screenbuffers[gl_drawbuffer].memP, 
							(xx+xoff)|(segment_width<<16), (yy+yoff)|(1<<16),
							(void *)imgbuf );
		}

		// With a "long" pointer, this assumes that the pixels are 32-bit
		theScanline += imgwid;
	}
}

////////////////////////////////////////////////////////////////////////////
// Clear the screen... Divide into 8x8 segments
////////////////////////////////////////////////////////////////////////////

void clearscreen(mmlDisplayPixmap *scrn)
{
long x, y;

    for (x = 0; x < scrn->wide; x += 8)
	{
		for (y = 0; y < scrn->high; y += 8)
		{
			_raw_plotpixel(scrn->dmaFlags, scrn->memP, (8<<16)|x, (8<<16)|y, clr_black);
		}
    }
}

